<?php

/**
 * @package Agnosia_Bootstrap_Carousel
 * @version 0.1
 */

/*
Plugin Name: Agnosia Bootstrap Carousel
Plugin URI: http://wordpress.org/extend/plugins/agnosia-bootstrap-carousel/
Description: Hooks the <code>[gallery]</code> shortcode with attribute <code>type="carousel"</code> in order to show a <a href="http://twitter.github.io/bootstrap/javascript.html#carousel" target="_blank">Bootstrap Carousel</a> based on the selected images and their titles and descriptions. <strong>Very important:</strong> this plugin assumes either your theme includes the necessary Bootstrap Javascript and CSS files to display the carousel properly, or that you have included the files on your own. It will not include the files for you, so if they are not present, the carousel will not work.
Author: Andr&eacute;s Villarreal
Version: 0.1
*/

remove_shortcode( 'gallery', 'gallery_shortcode' ); /* Remove original shortcode */
add_shortcode( 'gallery', 'agnosia_bootstrat_carousel_gallery_shortcode' ); /* Add custom shortcode */



function agnosia_bootstrat_carousel_gallery_shortcode( $attr ) {

	/* Validate for necessary data */
	if ( isset( $attr['ids'] ) and isset( $attr['type'] ) and $attr['type'] == 'carousel' ) :

		/* Define data by given attributes. */
		$ids = $attr['ids'];
		$name = isset( $attr['name'] ) ? sanitize_title( $attr['name'] ) : 'agnosia-bootstrap-carousel' ; /* Any name. String will be sanitize to be used as HTML ID. Recomended when you want to have more than one carousel in the same page. Default: agnosia-bootstrap-carousel. */
		$indicators = isset( $attr['indicators'] ) ? $attr['indicators'] : 'before-inner' ; /* Accepted values: before-inner, after-inner, after-control, false. Default: before-inner. */		
		$control = isset( $attr['control'] ) ? $attr['control'] : 'true' ; /* Accepted values: true, false. Default: true. */
		$interval = isset( $attr['interval'] ) ? $attr['interval'] : 5000 ; /* The amount of time to delay between automatically cycling an item. If false, carousel will not automatically cycle. */
		$pause = isset( $attr['pause'] ) ? $attr['pause'] : 'hover' ; /* Pauses the cycling of the carousel on mouseenter and resumes the cycling of the carousel on mouseleave. */

		/* Obtain HTML. */
		$output = agnosia_bootstrat_carousel_get_html_from( $ids , $name , $indicators , $control , $interval , $pause );

	/* If attributes could not be validated, execute default gallery shortcode function */
	else : $output = gallery_shortcode( $attr ) ;

	endif;

	return $output;

}



function agnosia_bootstrat_carousel_get_html_from( $ids , $name , $indicators , $control , $interval , $pause ) {

	/* Obtain posts array by given ids. Then construct HTML. */

	$images = agnosia_bootstrat_carousel_make_array( $ids );

	$output = '';

	if ( is_array( $images ) and !empty( $images ) ) :

		$posts = array();

		foreach ( $images as $image_id ) :

			$posts[] = get_post( intval( $image_id ) , ARRAY_A );

		endforeach;

		if ( is_array( $posts ) and !empty( $posts ) ) :

			$output = agnosia_bootstrat_carousel_make_html_from( $posts , $name , $indicators , $control , $interval , $pause );

		endif;

	endif;

	return $output;

}



function agnosia_bootstrat_carousel_make_html_from( $posts , $name , $indicators , $control , $interval , $pause ) {

	/* The important stuff happens here! */

	/* Initialize carousel HTML. */
	$output = '<div id="' . $name . '" class="carousel slide">';

	/* Try to obtain indicators before inner. */
	$output .= ( $indicators == 'before-inner' ) ? agnosia_bootstrat_carousel_make_indicators_html_from( $posts , $name ) : '' ;

	/* Initialize inner. */
	$output .= '<div class="carousel-inner">';

	/* Start counter. */
	$i = 0;

	/* Process each item into $posts array and obtain HTML. */
	foreach ( $posts as $post ) :

		if ( $post['post_type'] == 'attachment' ) : /* Make sure to include only attachments into the carousel */

			$image = wp_get_attachment_image_src( $post['ID'] , 'full' );

			$class = ( $i == 0 ) ? 'active ' : '';

			$output .= '<div class="' . $class . 'item" data-slide-no="' . $i . '">';

			$output .= '<img alt="' . $post['post_title'] . '" src="' . $image[0] . '" />';

			$output .= '<div class="carousel-caption">';

			$output .= '<h4>' . $post['post_title'] . '</h4>';
			$output .= $post['post_excerpt'];

			$output .= '</div>';

			$output .= '</div>';

			$i++;

		endif;

	endforeach;

	/* End inner. */
	$output .= '</div>';

	/* Try to obtain indicators after inner. */
	$output .= ( $indicators == 'after-inner' ) ? agnosia_bootstrat_carousel_make_indicators_html_from( $posts , $name ) : '' ;

	$output .= ( $control != 'false' ) ? agnosia_bootstrat_carousel_make_control_html_with( $name ) : '' ;

	/* Try to obtain indicators after control. */
	$output .= ( $indicators == 'after-control' ) ? agnosia_bootstrat_carousel_make_indicators_html_from( $posts , $name ) : '' ;

	/* End carousel HTML. */
	$output .= '</div>';

	/* Obtain javascript for carousel. */
	$output .= '<script type="text/javascript">// <![CDATA[
jQuery(document).ready( function() { jQuery(\'#' . $name . '\').carousel( { interval : ' . $interval . ' , pause : "' . $pause . '" } ); } );
// ]]></script>';

	return $output;

}


/* Obtain indicators from $posts array. */
function agnosia_bootstrat_carousel_make_indicators_html_from( $posts , $name ) {

	$output = '<ol class="carousel-indicators">';

	$i = 0;

	foreach ( $posts as $post ) :

		if ( $post['post_type'] == 'attachment' ) : /* Make sure to include only attachments into the carousel */

			$class = ( $i == 0 ) ? 'active' : '';

			$output .= '<li data-target="#' . $name . '" data-slide-to="' . $i . '" class="' . $class . '"></li>';

			$i++;

		endif;

	endforeach;

	$output .= '</ol>';

	return $output;

}


/* Obtain control links. */
function agnosia_bootstrat_carousel_make_control_html_with( $name ) {

	$output = '<a class="carousel-control left" href="#' . $name . '" data-slide="prev">&lsaquo;</a>';
	$output .= '<a class="carousel-control right" href="#' . $name . '" data-slide="next">&rsaquo;</a>';

	return $output;

}



/* Obtain array of id given comma-separated values in a string. */
function agnosia_bootstrat_carousel_make_array( $string ) {

	$array = explode( ',' , $string );
	return $array;

}



?>
